dc.api = {
  /**
   * 短信前置-获取图片验证码
   * @param {String} phone 手机号码
   */
  getImageCode: function () {
    return dc.request({
      url: "sms/loadImageCode",
    });
  },

  /**
   * 获取短信验证码
   * @param {String} phone 手机号码
   * @param {Number} imageCode 4位数字的图片验证码
   */
  sendSmscode: function (phone, imageCode) {
    return dc.request({
      url: "sms/sendSms",
      method: "post",
      data: {
        phone: phone,
        imageCode: imageCode,
        type: 32,
      },
    });
  },

  /**
   * 短信验证码登录
   * @param {String} phone 手机号码
   * @param {Number} smsCode 6位数字的短信验证码
   */
  smscodeLogin: function (phone, smsCode) {
    return dc.request({
      url: "login/sms",
      method: "post",
      data: {
        phone: phone,
        smsCode: smsCode,
        type: 32,
      },
    });
  },

  /**
   * 账号密码登录
   * @param {String} phone 账号
   * @param {String} password 密码
   */
  passwordLogin: function (phone, password) {
    return dc.request({
      url: "login/login",
      method: "post",
      data: {
        phone: phone,
        password: password,
      },
    });
  },

  // 自动登录
  autoLogin: function () {
    return dc.request({
      url: "login/autoLogin",
      method: "post",
    });
  },

  /**
   * 微信登录
   * @param {String} openid 微信授权用户唯一标识
   * @param {String} accessToken 微信接口调用凭证
   */
  weChatLogin: function (openid, accessToken) {
    return dc.request({
      url: "login/wechatLogin",
      data: {
        openid: openid,
        accessToken: accessToken,
      },
    });
  },

  /**
   * 用户注册
   * @param {String} phone 账号
   * @param {String} password 密码
   * @param {String} smsCode 	短信验证码
   */
  loginRegister: function (phone, password, smsCode) {
    return dc.request({
      url: "login/register",
      method: "post",
      data: {
        phone: phone,
        password: password,
        smsCode: smsCode,
      },
    });
  },

  /**
   * 获取首页公开课列表
   * @param {Number} professionId 职称ID
   */
  getIndexOpenClassList: function (professionId) {
    return dc.request({
      url: "main/freeLiveList/" + professionId,
    });
  },

  /**
   * 获取公开课列表
   * @param {Number} professionId 职称ID
   * @param {Number} page 当前页码
   */
  getOpenClassList: function (professionId, page) {
    return dc.request({
      url: "main/freeLiveList/" + professionId + "/" + (page || 1),
    });
  },

  /**
   * 公开课预约
   * @param {Number} professionId 职称Id
   * @param {Number} status 预约或者取消预约标识：1、预约；2、取消预约
   * @param {Number} liveDetailId 直播详细id
   */
  reserveOpenclass: function (professionId, status, liveDetailId) {
    return dc.request({
      url: "https://cj.tibaitong.com/api/push/prepareCourse",
      data: {
        professionId: professionId,
        status: status,
        liveDetailId: liveDetailId,
      },
    });
  },

  /**
   * 修改昵称
   * @param {String} nickname 要修改的昵称
   */
  changeNick: function (nickname) {
    return dc.request({
      url: "user/changeNick",
      method: "post",
      data: {
        nick: nickname,
      },
    });
  },

  /**
   * 修改密码
   * @param {String} oldPwd 原密码
   * @param {String} newPwd 新密码
   */
  changePassword: function (oldPwd, newPwd) {
    return dc.request({
      url: "user/changePassword",
      data: {
        oldPwd: oldPwd,
        newPwd: newPwd,
      },
    });
  },

  /**
   * 切换职称
   * @param {Number} professionId 职称Id
   */
  changeProfession: function (professionId) {
    return dc.request({
      url: "login/changeProfession",
      data: {
        professionId: professionId,
      },
    });
  },

  /**
   * 切换课程
   * @param {Number} courseId 课程Id
   */
  changeCourse: function (courseId) {
    return dc.request({
      url: "login/changeCourse",
      data: {
        courseId: courseId,
      },
    });
  },

  /**
   * 获取课程的用户数据
   * @param {Number} courseId 课程id
   * @param {Number} sectionType 章节类型
   */
  getCourseDataByUser: function (courseId) {
    return dc.request({
      url: `course/getLearningSummary/${courseId}`,
    })
  },

  /**
   * 获取用户在某一课程下已做题总量
   * @param {Number} courseId 课程id
   * @param {Number} sectionType 章节类型
   */
  getDidCount: function (courseId) {
    return dc.request({
      url: `userQuestion/getUserCourseQuestionsNum?courseId=${courseId}`,
    })
  },

  /**
   * 获取考试倒计时
   * @param {Number} professionId 职称id
   */
   getExamCountDown: function (professionId) {
    return dc.request({
      url: `general/getExaminationDate?professionId=${professionId}`,
    })
  },


  /**
   * 获取错题打印是否有权限，和是否开放
   * @param {Number} courseId 课程Id
   */
  getErrorAuth: function (courseId) {
    return dc.request({
      url: "section/getErrorPrintInfo",
      data: {
        courseId: courseId,
      },
    });
  },

  /**
   * 错题打印-错题点评
   * @param {Number} professionId 职称Id
   * @param {Number} courseId 课程Id
   */
  getErrorComment: function (professionId, courseId) {
    return dc.request({
      url: "section/getErrorComment",
      data: {
        professionId: professionId,
        courseId: courseId,
      },
    });
  },

  /**
   * 获取题库类型列表
   * @param {Number} courseId 课程Id
   */
  getTikuType: function (courseId) {
    return dc.request({
      url: "section/findSectionType",
      data: {
        courseId: courseId,
      },
    });
  },

  /**
   * 导出试题
   * @param {Number} sectionId 章节id
   * @param {Number} exportType 导出类型
   */
  exportQuestion: function (sectionId, exportType) {
    return dc.request({
      url: "export/question",
      data: {
        sectionId: sectionId,
        exportType: exportType,
      },
    });
  },

  /**
   * 判断题库章节类型权限
   * @param {Number} courseId 课程id
   * @param {Number} sectionType 章节类型
   */
  checkTikuAuth: function (courseId, sectionType) {
    return dc.request({
      url: "course/isHasAuthOfSection",
      data: {
        courseId: courseId,
        type: sectionType,
      },
    });
  },

  /**
   * 获取直播列表
   * @param {Number} courseId 课程id
   */
  getLiveList: function (courseId) {
    return dc.request({
      url: "live/findLiveList/" + courseId,
    });
  },

  /**
   * 判断视频类型权限
   * @param {Number} courseId 课程id
   * @param {Number} videoType 视频类型
   */
  checkVideoAuth: function (courseId, videoType) {
    return dc.request({
      url: "course/isHasAuthOfVideo",
      data: {
        courseId: courseId,
        type: videoType,
      },
    });
  },

  /**
   * 检查直播权限
   * @param {Number} liveDetailId 直播详细id
   */
  checkLiveAuth: function (liveDetailId) {
    return dc.request({
      url: "live/checkLiveAuth/" + liveDetailId,
    });
  },

};
